// 002 u_c_cgi_base_forms
// 18 may 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_cgi_base_forms;
  interface
    uses Classes, u_c_cgi_basic_object;

    type c_base_form= class(c_basic_object)
                        m_name: String;

                        m_content_length: Integer;

                        m_request_method: String;
                        m_remote_address_string: String;
                        m_raw_answer_string: String;

                        m_query_string: String;

                        Constructor create_base_form(p_name: String);

                        procedure get_form_result;
                        function f_display_form(p_domain: String): String;

                        Destructor Destroy; Override;
                    end; // c_base_form

  implementation
    uses SysUtils, Windows
        , u_cgi_dir
        , u_c_cgi_log
        , u_cgi_helpers // -- convert special characters
        ;

    const k_return= Chr(13);
          k_line_feed= Chr(10);

    // -- c_base_form

    Constructor c_base_form.create_base_form(p_name: String);
      begin
        Inherited create_basic_object(p_name);
      end; // create_base_form

    // -- form-specific

    procedure c_base_form.get_form_result;
        // -- get some values from the environment

      function f_get_environment_value(p_key: pChar): String;
        const k_max= 255;
        var l_environment_value: String[k_max];
            l_size: Integer;
        begin
          l_size:= GetEnvironmentVariable(p_key, @ l_environment_value[1], k_max);
          l_environment_value[0]:= chr(l_size);
          Result:= l_environment_value;
        end; // f_get_environment_value

      var // -- why ? can have several identical in a row ?
          l_previous_environment_value: String;

      procedure check_environment_key(p_key: pChar);
        var l_environment_value: String;
        begin
          l_environment_value:= f_get_environment_value(p_key);

          if l_environment_value<> l_previous_environment_value
            then begin
                if p_key= 'REQUEST_METHOD'
                  then m_request_method:= l_environment_value
                  else
                    if p_key= 'REMOTE_ADDR'
                      then m_remote_address_string:= l_environment_value;
              end;

          l_previous_environment_value:= l_environment_value;
        end; // check_environment_key

      procedure read_content_length;
        var l_content: String;
        begin
          l_content:= f_get_environment_value('CONTENT_LENGTH');
          If Length(l_content)> 0
            then m_content_length:= StrToInt(l_content);
        end; // read_content_length

      procedure read_answer;
        var l_index: Integer;
        begin
          if (UpperCase(m_request_method)= 'POST') and (m_content_length> 0)
            then
              begin
                // -- +1 for last & added by us
                SetLength(m_raw_answer_string, m_content_length);

                // -- read from StdIn
                for l_index:= 1 to m_content_length do
                  Read(m_raw_answer_string[l_index]);

                // -- add ending "&" (sentinell)
                m_raw_answer_string:= m_raw_answer_string+ '&';
              end // post and content-length
            else m_raw_answer_string:= '*** no_post or 0 length';
        end; // read_answer

      begin // get_form_result
        l_previous_environment_value:= '';

        // -- first get some values from the environment
        check_environment_key('SERVER_PORT');
        check_environment_key('SERVER_PROTOCOL');
        // -- GET, POST, PUT
        check_environment_key('REQUEST_METHOD');
        // -- the file requested ("filename=u_c_log.zip")
        check_environment_key('QUERY_STRING');
        // -- "192.0.0.3"
        check_environment_key('REMOTE_ADDR');
        // -- "plus_facile.com"
        check_environment_key('REMOTE_HOST');

        // -- if has the length, fetch the additional
        read_content_length;
        read_answer;

        // -- not necessary
        m_query_string:= f_get_environment_value('QUERY_STRING');
      end; // get_form_result

    function c_base_form.f_display_form(p_domain: String): String;
(*
      const k_new_line= chr(13)+ Chr(10);

      function f_non_empty_string(p_key: t_string_position; p_title: String): String;
        begin
          Result:= f_read_string(p_key);
          if Result<> ''
            then Result:= k_new_line+ '                 '+ p_title+ ': '+ Result;
        end; // f_non_empty_string

      function f_non_empty_long_string(p_key: t_string_position; p_title: String): String;
        begin
          Result:= f_read_long_string(p_key);

          if Result<> ''
            then Result:= k_new_line+ '                 '+ p_title+ ': '
                + f_convert_special_characters(Result);
        end; // f_non_empty_long_string

      function f_remote(p_remote: String): String;
        begin
          Result:= k_new_line+ '                 Rem_ad: '+ p_remote+ ' '+ p_domain;
        end; // f_remote

      function f_format_answer(p_answer: String): String;
         // -- AAA=aaa&BBB=bbb&...

        function f_extract_string(p_start, p_end: Integer): String;
          var l_length: Integer;
              l_index: Integer;
          begin
            l_length:= p_end+ 1- p_start;
            if l_length> 0
              then begin
                  SetLength(Result, l_length);
                  Move(p_answer[p_start], Result[1], l_length);

                  // -- remove RET, LF
                  for l_index:= 1 to l_length do
                    if (Result[l_index]= k_return)
                        or (Result[l_index]= k_line_feed)
                      then Result[l_index]:= ' ';
                end
              else Result:= '';
          end; // f_extract_string

        var l_index, l_length, l_start_index: Integer;
            l_title, l_answer: String;

        begin // f_format_answer
          l_index:= 1; l_length:= Length(p_answer);
          // display(' >'+ p_answer+ '<');

          while l_index<= l_length do
          begin
            l_start_index:= l_index;
            while (l_index<= l_length) and (p_answer[l_index]<> '=') do
              Inc(l_index);
            l_title:= f_extract_string(l_start_index, l_index- 1);

            // -- skip =
            Inc(l_index);

            l_start_index:= l_index;
            while (l_index<= l_length) and (p_answer[l_index]<> '&') do
              Inc(l_index);
            l_answer:= f_extract_string(l_start_index, l_index- 1);

            if l_answer<> ''
              then begin
                  Result:= Result+ k_new_line+ Format('%-12s ', [l_title])+ l_answer;
                end;

            // -- skip &
            Inc(l_index);
          end; // while l_index<= l_length
        end; // f_format_answer

      begin // f_display_form
        with m_form do
          Result:= Format('%4d %2d %3d %4d ', [m_id, m_length, m_page, m_content_length])
            + FormatDateTime('yyyy/mm/dd hh:nn:ss:zzz ', m_date)
            + f_non_empty_string(m_server_port,     'Port  ')
            + f_non_empty_string(m_server_protocol, 'Proto ')
            + f_non_empty_string(m_query_string,    'Query ')
            + f_non_empty_string(m_request_method,  'Method')
            + f_non_empty_string(m_remote_host,     'Remote')
            // + f_non_empty_string(m_remote_address,  'Rem_ad')
            + f_remote(f_read_string(m_remote_address))

            + f_format_answer(f_read_long_string(m_answer_string))
            ;
*)
      begin
      end; // f_display_form

    Destructor c_base_form.Destroy;
      begin
        Inherited;
      end; // Destroy

    begin // u_c_form
    end. // u_c_form
