// 002 u_cgi_helpers
// 18 may 2005

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_cgi_helpers;
  interface
    uses Classes;

    function f_cgi_exe_path: String;

    // -- environment handling

    function f_request_method: String;
    function f_content_length: Integer;

    function f_raw_request_string: String;
    function f_convert_special_characters(p_string: String): String;
    function f_replace_double_quote(p_string: String): String;
    function f_c_parameter_list: tStringList;

    procedure extract_key_values(p_raw_request: String;
        p_c_key_value_list: tStringList);

  implementation
    uses SysUtils, Windows
        , u_cgi_environment;

    // -- server pathes

    function f_cgi_exe_path: String;
      var l_size: Integer;
          l_cgi_exe_name_array: array[0..MAX_PATH] of Char;
          l_cgi_exe_name: String;
      begin
        l_size:= Windows.GetModuleFileName(hInstance, l_cgi_exe_name_array, MAX_PATH);
        SetString(l_cgi_exe_name, l_cgi_exe_name_array, l_size);
        Result:= ExtractFilePath(l_cgi_exe_name);
      end; // f_cgi_exe_path

    // -- request handling

    function f_request_method: String;
      begin
        Result:= f_get_environment_value('REQUEST_METHOD');
      end; // f_request_method

    function f_content_length: Integer;
      var l_content_length: String;
      begin
        l_content_length:= f_get_environment_value('CONTENT_LENGTH');
        if l_content_length<> ''
          then Result:= StrToInt(l_content_length)
          else Result:= 0;
      end; // f_content_length

    var // avoid reading twice the parameters
        s_raw_request_string: String= '';

    function f_raw_request_string: String;
      // -- CAUTION: only read values ONCE (else hangs)
      var l_request_method: String;
          l_content_length: Integer;
          l_index: Integer;
      begin
        if s_raw_request_string<> ''
          then Result:= s_raw_request_string
          else begin
              l_request_method:= f_request_method;
              l_content_length:= f_content_length;

              // -- now read the question
              if (l_request_method= 'POST') and (l_content_length> 0)
                then begin
                    // -- +1 for last & added by us
                    SetLength(Result, l_content_length);

                    // -- read from StdIn
                    for l_index:= 1 to l_content_length do
                      Read(Result[l_index]);

                    s_raw_request_string:= Result;
                  end
                else Result:= '';
            end;
      end; // f_raw_request_string

    function f_convert_special_characters(p_string: String): String;
        // -- convert
        // --   + to space
        // --   $xx to chr($xx)
      var l_hex_string: ShortString;
          l_index: Integer;
      begin
        l_index:= 1;
        Result:= '';

        while l_index<= Length(p_string) do
        begin
          if p_string[l_index]= '+'
            then // -- change the + back to spaces
                 Result:= Result+ ' '
            else
              if p_string[l_index]= '%'
                then
                  begin
                    // -- change specials %26 into &
                    l_hex_string:= '$00';
                    l_hex_string[2]:= p_string[l_index+ 1];
                    l_hex_string[3]:= p_string[l_index+ 2];
                    Result:= Result+ Chr(StrToInt(l_hex_string));
                    Inc(l_index, 2);
                  end
                else Result:= Result+ p_string[l_index];

          Inc(l_index);
        end; // while
      end; // f_convert_special_characters

    function f_replace_double_quote(p_string: String): String;
      var l_index: Integer;
      begin
        Result:= '';
        for l_index:= 1 to Length(p_string) do
          if p_string[l_index]= '"'
            then begin
                Result:= Result+ '&quot;';
              end
            else Result:= Result+ p_string[l_index];
      end; // f_replace_double_quote

    function f_c_parameter_list: tStringList;
      var l_raw_request_string, l_request_string: String;
          l_index: Integer;
          l_parameter: String;
      begin
        Result:= tStringList.Create;

        l_raw_request_string:= f_raw_request_string;
        l_request_string:= f_convert_special_characters(l_raw_request_string);

        // -- format: xxx=yyy&xxx=yyy
        l_parameter:= '';
        for l_index:= 1 to Length(l_request_string) do
        begin
          if l_request_string[l_index]= '&'
            then begin
                Result.Add(l_parameter);
                l_parameter:= '';
              end
            else l_parameter:= l_parameter+ l_request_string[l_index];
        end; // for

        // -- add the last one
        if l_parameter<> ''
          then Result.Add(l_parameter);
      end; // f_c_parameter_list

    procedure extract_key_values(p_raw_request: String;
        p_c_key_value_list: tStringList);
        // -- convert
        // --   + to space
        // --   $xx to chr($xx)
      var l_hex_string: ShortString;
          l_index: Integer;
          l_string: String;
      begin
        l_index:= 1;
        l_string:= '';

        while l_index<= Length(p_raw_request) do
        begin
          case p_raw_request[l_index] of
            '+' : l_string:= l_string+ ' ';
            '%' : begin
                    // -- change specials %26 into &
                    l_hex_string:= '$00';
                    l_hex_string[2]:= p_raw_request[l_index+ 1];
                    l_hex_string[3]:= p_raw_request[l_index+ 2];
                    l_string:= l_string+ Chr(StrToInt(l_hex_string));
                    Inc(l_index, 2);
                  end;
            '&' : begin
                    // p_c_key_value_list.Add(Trim(l_string));
                    p_c_key_value_list.Add(l_string);
                    l_string:= '';
                  end;
            else
              l_string:= l_string+ p_raw_request[l_index];
          end; // case

          Inc(l_index);
        end; // while

        // -- last value
        // --   should not happen if appended & to the request
        if l_string<> ''
          then p_c_key_value_list.Add(Trim(l_string));
      end; // extract_key_values

    begin
    end.

